#include <iostream>
#include <vector>

#include "TROOT.h"
#include "TStyle.h"
#include "TCanvas.h"
#include "TF1.h"
#include "TF2.h"
#include "TH1.h"
#include "TH2.h"
#include "TMath.h"

const Int_t nNodes=2;
const Int_t nVars=2;

Double_t activation(Double_t x){
   return 1./(1+TMath::Exp(-x));
}


Double_t mlp(Double_t w1[nNodes][nVars], Double_t w2[nNodes], Double_t x[nVars], TH2D* *fn){
   
   Double_t result=0;
   Double_t arg;
   for (Int_t iNode=0; iNode < nNodes; iNode++){
      arg=0;
      for (Int_t iVar=0; iVar < nVars; iVar++){
         arg+=w1[iNode][iVar]*x[iVar];
      }
      fn[iNode]->Fill(x[0],x[1],activation(arg));
      result += w2[iNode] * activation(arg);
   }
   result = activation(result);
   return result;
}

void miniMLP2()
{
   gROOT->SetStyle("Plain");
   gStyle->SetOptStat(0);
 
   TH2D *fn[nNodes+1]; // use the last one as 'output node'
   for (Int_t i=0; i<=nNodes; i++){
      fn[i]=new TH2D(Form("fn%d",i),"Node1",100,-10,10,100,-10,10);
      fn[i]->SetTitle(Form("Node%d",i));
      fn[i]->GetXaxis()->SetTitle("X_{1}");
      fn[i]->GetYaxis()->SetTitle("X_{2}");
      fn[i]->GetXaxis()->SetTitleOffset(1.5);
      fn[i]->GetXaxis()->SetTitleSize(0.06);
      fn[i]->GetYaxis()->SetTitleOffset(1.5);
      fn[i]->GetYaxis()->SetTitleSize(0.05);
      fn[i]->GetXaxis()->SetLabelSize(0.04);
      fn[i]->GetYaxis()->SetLabelSize(0.04);
      fn[i]->GetXaxis()->SetNdivisions(305);
      fn[i]->GetYaxis()->SetNdivisions(305);
   }



   Double_t x[nVars]; 
   Double_t w1[nNodes][nVars];
   Double_t w2[nNodes];

   w1[0][0] = -1;
   w1[0][1] =  1;

   w1[1][0] = -4;
   w1[1][1] = -1;

   w2[0] =  1;
   w2[1] =  1;


   Double_t x2Step=0.2,x1Step=0.2;
   x[0]=-10.+x1Step/2.;
   x[1]=-10.+x2Step/2.;

   // loop over x,y data to fill a histogram allowing the visualisation of the network 'function'
   for (Int_t iX1=0; iX1<100; iX1++){ 
      x[1]=-10. +  x2Step/2.;
      for (Int_t iY=0; iY<100; iY++){
         // the actual calculation of the 'network' output
         Double_t mlpResult = mlp(w1,w2,x,fn);

         fn[nNodes]->Fill(x[0],x[1],mlpResult);
         x[1]+=x2Step;
      }         
      x[0]+=x1Step;
   }

   TCanvas *c=new TCanvas("c","",900,900);
   c->Divide(2,2);
   c->cd(1);
   fn[0]->Draw("surf");

   c->cd(2);
   fn[1]->Draw("surf");

   c->cd(3);
   fn[nNodes]->Draw("surf");

   c->cd(4);
   Double_t contours[1]={0.55};
   fn[nNodes]->SetContour(1,contours);
   fn[nNodes]->Draw("CONT1");


}
